/*
 * Assemble Plugin: toc
 * https://bitbucket.org/goldcome/assemble-plugin-toc
 *
 * Copyright (c) 2014 GoldCome, contributors
 * Licensed under the MIT license.
 */

'use strict';

module.exports = function (grunt) {

    // Project configuration.
    grunt.initConfig({

        pkg: grunt.file.readJSON('package.json'),

        jshint: {
            options: {
                curly: true,
                eqeqeq: true,
                immed: true,
                latedef: true,
                newcap: true,
                noarg: true,
                sub: true,
                undef: true,
                boss: true,
                eqnull: true,
                node: true
            },
            all: ['Gruntfile.js', 'index.js']
        },

        assemble: {
            options: {
                assets: 'test/assets',
                plugins: ['./index.js'],
                layoutdir: 'test/fixtures/layouts',
                layoutext: '.hbs',
                layout: 'default'
            },
            toc: {
                files: [
                    {
                        expand: true,
                        cwd: 'test/fixtures/pages',
                        src: ['**/*.hbs'],
                        dest: 'test/actual/toc'
                    }
                ]
            },
            toc_option: {
                options: {
                    toc: {
                        boxid: 'toc-content',
                        tocid: 'toc-test',
                        listid: 'listid',
                        listclass: 'listclass',
                        hclassprefi: 'toc-test-h',
                        hnameprefix: 'toc-test-name'
                    }
                },
                files: [
                    {
                        expand: true,
                        cwd: 'test/fixtures/pages',
                        src: ['**/*.hbs'],
                        dest: 'test/actual/toc_option'
                    }
                ]
            }
        },

        // Before generating new files, remove any files from previous build.
        clean: {
            actual: ['test/actual/**']
        },

        //Start a server
        connect: {
            server: {
                options: {
                    port: 9000,
                    base: '.',
                    keepalive: true
                }
            }
        }
    });

    // These plugins provide necessary tasks.
    grunt.loadNpmTasks('assemble');
    grunt.loadNpmTasks('grunt-readme');
    grunt.loadNpmTasks('grunt-contrib-clean');
    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-connect');

    // By default, lint and run all tests.
    grunt.registerTask('default', ['jshint', 'clean', 'assemble']);
};
