/*
 * assemble-plugin-toc
 * Copyright (c) 2012 GoldCome, contributors.
 * Licensed under the MIT license.
 */

var options = {
    stage: 'render:post:page'
};

var cheerio = require('cheerio');

/**
 * Toc Plugin
 * @param  {Object}   params
 * @param  {Function} callback
 */
module.exports = function (params, callback) {
    'use strict';

    var content = params.content;

    var opts = params.assemble.options;
    opts.toc = opts.toc || {};

    // boxid是要进行提取目录的容器，默认全局
    // tocid是增加html容器id，默认toc
    // listid是定义列表id，默认toc-list
    // listclass定义列表的类，默认空
    // hclassprefix新生成列表标题class前缀，默认toc-h
    // hnameprefix原文里标题name前缀，用来做描点，默认toc-name
    var boxid = '#' + (opts.toc.boxid || '');
    var tocid = '#' + (opts.toc.tocid || 'toc');
    var listid = opts.toc.listid || 'toc-list';
    var listclass = opts.toc.listclass || '';
    var hclassprefix = opts.toc.hclassprefix || 'toc-h';
    var hnameprefix = opts.toc.hnameprefix || 'toc-name';
    var li = opts.toc.li ? (' class="' + opts.toc.li + '"') : '';

    // 加载当前页内容
    var $ = cheerio.load(content);
    var toc = cheerio.load('<ul id=' + listid + ' class="' + listclass + '"></ul>');

    var hid = 1;
    //构建选择器函数，实际上是字符串拼接
    //参数：h1 h2 h3.....
    function buildSelector(){
        var result = '';
        for(var i=0;i<arguments.length;i++) {
            if(boxid === '#'){
                result = result + arguments[i];
            }
            else{
                result = result + boxid + ' ' + arguments[i];
            }
            if ( i<arguments.length - 1) {
                result = result + ',';
            }
        }
        //console.log(result);
        return result;
    }
    var headings = $(buildSelector('h1','h2','h3','h4','h5'));
    /* 相关说明
     *   e代表本对象，==this
     *   e.attribs.name获取name属性值，== $(e).attr('name');前者无法访问父或子对象，后者可以通过parent()和children()方法访问
     *   e.name，获取包络标签，如h1 h2 h3 p等，$(e)确无法获取
     */

    //增加name属性用来描点，并给予规则toc-name + 数字命名（因为原来id如果汉字是无法提取id）
    //这里增加一部分html，不直接在h中添加id，而另加一部分，加上一个链接样式，bootstrap链接描点样式
    headings.map(function (i, e) {
        if (e.attribs.id) {
            var anchor = [
                '<a href="#' + hnameprefix + hid + '" name="'  + hnameprefix + hid + '" class="anchor">',
                '  <span class="anchor-target" ></span>',
                '  <span class="glyphicon glyphicon-link"></span>',
                '</a>\n'
            ].join('\n');
            $(this).append(anchor);
            $(this).removeAttr('id');
            hid++;
        }
    });
    //会用正则去掉层级，为了区分不同标题样式，加上不同类名，这里默认是toc-h + 数字格式
    headings.map(function (i, e) {
        var text = $(e).text().trim();
        var link = $(e).children('a.anchor').attr('name');
        var depth = parseInt(e.name.replace(/h/gi, ''), 10);
        var arr = new Array(depth);
        var level = arr.join('<li><ul>') + '<li class=' + '"' + hclassprefix + depth + '"' + '><a href="#' + link + '">' + text + '</a></li>' + arr.join('</ul></li>');
        toc('#' + listid).append(level);
    });
    $(tocid).append(toc.html()
        //去除层级正则替换
        .replace(/(<li>\s*<ul>\s*)+/g, '')
        .replace(/(<\/ul>\s*<\/li>\s*)+/g, '')
    );

    params.content = $.html();
    callback();
};

module.exports.options = options;