#rsync配置和使用

###rsync概括

Rsync 是一个远程数据同步工具，可通过 LAN/WAN 快速同步多台主机间的文件。 Rsync 本来是用以取代 rcp 的一个工具，它当前由 rsync.samba.org 维护。 Rsync 使用所谓的“ Rsync 演算法 ”来使本地和远程两个主机之间的文件达到同步，这个算法只传送两个文件的不同部分，而不是每次都整份传送，因此速度相当快。运行 Rsync server 的机器也叫 backup server ，一个 Rsync server 可同时备份多个 client 的数据；也可以多个 Rsync server 备份一个 client 的数据。

Rsync 可以搭配 rsh 或 ssh 甚至使用 daemon 模式。 Rsync server 会打开一个 873 的服务通道 (port) ，等待对方 Rsync 连接。连接时， Rsync server 会检查口令是否相符，若通过口令查核，则可以开始进行文件传输。第一次连通完成时，会把整份文件传输一次，下一次就只传送二个文件之间不同的部份。

Rsync 支持大多数的类 Unix 系统，无论是 Linux 、 Solaris 还是 BSD 上都经过了良好的测试。此外，它在 windows 平台下也有相应的版本，比较知名的有 cwRsync 和 Sync2NAS 。


###Rsync 的基本特点如下

* 可以镜像保存整个目录树和文件系统；

* 可以很容易做到保持原来文件的权限、时间、软硬链接等；

* 无须特殊权限即可安装；

* 优化的流程，文件传输效率高；

* 可以使用 rcp 、 ssh 等方式来传输文件，当然也可以通过直接的 socket 连接；

* 支持匿名传输。

###核心算法介绍：

假定在名为α和β的两台计算机之间同步相似的文件 A 与 B ，其中α对文件 A 拥有访问权，β对文件 B 拥有访问权。并且假定主机α与β之间的网络带宽很小。那么 rsync 算法将通过下面的五个步骤来完成：

* β将文件 B 分割成一组不重叠的固定大小为 S 字节的数据块。最后一块可能会比S小。

* β对每一个分割好的数据块执行两种校验：一种是 32 位的滚动弱校验，另一种是 128 位的 MD4 强校验。

* β将这些校验结果发给α。

* α通过搜索文件 A 的所有大小为 S 的数据块 ( 偏移量可以任选，不一定非要是 S 的倍数 ) ，来寻找与文件 B 的某一块有着相同的弱校验码和强校验码的数据块。这项工作可以借助滚动校验的特性很快完成。

* α发给β一串指令来生成文件 A 在β上的备份。这里的每一条指令要么是对文件 B 经拥有某一个数据块而不须重传的证明，要么是一个数据块，这个数据块肯定是没有与文件 B 的任何一个数据块匹配上的。

##指令说明

###命令格式：

```bash

rsync [option] 源路径 目标路径

```

###[option]：

* a:使用archive模式，等于-rlptgoD，即保持原有的文件权限

* z:表示传输时压缩数据

* v:显示到屏幕中

* e:使用远程shell程序（可以使用rsh或ssh）

* --delete:精确保存副本，源主机删除的文件，目标主机也会同步删除

* --include=PATTERN:不排除符合PATTERN的文件或目录

* --exclude=PATTERN:排除所有符合PATTERN的文件或目录

* --password-file:指定用于rsync服务器的用户验证密码

***以上是常用参数列表，完整列表见文章结尾***


####rsync 的命令格式可以为以下六种：

```bash

rsync [OPTION]... SRC DEST

rsync [OPTION]... SRC [USER@]HOST:DEST

rsync [OPTION]... [USER@]HOST:SRC DEST

rsync [OPTION]... [USER@]HOST::SRC DEST

rsync [OPTION]... SRC [USER@]HOST::DEST

rsync [OPTION]... rsync://[USER@]HOST[:PORT]/SRC [DEST]

```

对应于以上六种命令格式， rsync 有六种不同的工作模式：


* 拷贝本地文件。当 SRC 和 DES 路径信息都不包含有单个冒号 ":" 分隔符时就启动这种工作模式。

* 使用一个远程 shell 程序 ( 如 rsh 、 ssh) 来实现将本地机器的内容拷贝到远程机器。当 DST 路径地址包含单个冒号 ":" 分隔符时启动该模式。

* 使用一个远程 shell 程序 ( 如 rsh 、 ssh) 来实现将远程机器的内容拷贝到本地机器。当 SRC 地址路径包含单个冒号 ":" 分隔符时启动该模式。

* 从远程 rsync 服务器中拷贝文件到本地机。当 SRC 路径信息包含 "::" 分隔符时启动该模式。

* 从本地机器拷贝文件到远程 rsync 服务器中。当 DST 路径信息包含 "::" 分隔符时启动该模式。

* 列远程机的文件列表。这类似于 rsync 传输，不过只要在命令中省略掉本地机信息即可。 　

***需要注意的是，来源或目的路径最少要有一个是本地路径，如果忽略本地路径，则只会列出远端的文件列表。***

###使用说明

配置用于Rsync同步的远程主机，常见的有两种服务实现方式：

下面，我们两详细的说明两种服务实现方式：


第一，基于远程shell的程序，比如ssh;只需要安装Openssh-server，启动服务即可。特点：可以直接使用linux系统用户进行验证，很方便灵活。 这里主要是设计ssh使用，参照我的文章 ssh使用整理，***注意使用参数e和ssh就ok***

第二，其程序自身的“daemon”服务器模式。

####1.使用基于OPENSSH的服务方式是，远程主机（服务器端）不需要设置rsync，

```bash

rsync -ave ssh /cygdrive/g/Dropbox goldcome@192.168.1.109:/home/test/

```

####2,基于rsync的daemon模式

#####rsync服务器配置

```

#全局选项
strict modes =yes #是否检查口令文件的权限,如果设置yes，密码文件一定要是root:root而且是600权限

port = 873 #默认端口873

use chroot = no #禁止chroot(不可连接后切换用户)

list = yes #允许访问目录列表

## 下面文件是安装完RSYNC服务后自动生成的文件,当然也可以手动配置到指定路径，一般我是不理

#pid file = /var/run/rsyncd.pid # pid文件的存放

#lock file = /var/run/rsync.lock # 锁文件的存放位置

#log file = /var/log/rsyncd.log # 日志记录文件的存放

#motd file = /etc/rsyncd.motd # 欢迎文件存放位置

#模块选项

[test] # 这里是认证的模块名，在client端需要指定

max connections = 5 # 客户端最大连接数,默认0(没限制)

uid = root # 指定该模块传输文件时守护进程应该具有的uid(用户),必须是有读取path权限的用户、组

gid = root # 指定该模块传输文件时守护进程应该具有的gid(组),必须是有读取path权限的用户、组

path = /home/goldcome/test # 需要做备份的目录

ignore errors # 可以忽略一些无关的IO错误

read only = no # no客户端可上传文件,yes只读

write only = no # no客户端可下载文件,yes不能下载

hosts allow = * # 充许任何主机连接

#hosts deny = 10.5.3.77 # 禁止指定的主机连接

auth users = rsync # 认证的用户名，如果没有这行，则表明是匿名，用户随便设置，可以不是系统用户，和下面配对就行

secrets file = /home/goldcome/rsync.pwd # 指定认证口令文件位置

```

#####生成rsync密码文件

rsync:rsync #用户名:密码

在server端生成一个密码文件/home/goldcome/rsync.pwd

格式：帐号:密码 （每行一组，帐号和密码用:号分开）

***注意：密码文件的权限，是由rsyncd.conf里的参数strict modes =yes/no 来决定***


#####Rsync启动和停止

######Rsync 的启动

```bash

sudo rsync --daemon --config=/etc/rsyncd.conf

```

这里我只用守护进程方式：

注意这里要用root权限来启动，否则客户端连接会连不上


######Rsync重启

```
goldcome@goldcome-virtual-machine:~$ ps -ef | grep rsync (查找进程)

root 4080 3894 0 16:16 pts/2 00:00:00 sudo gvim -f /etc/rsyncd.conf

root 4081 4080 0 16:16 pts/2 00:00:06 gvim -f /etc/rsyncd.conf

root 4781 1 0 17:36 ? 00:00:00 rsync --daemon --config=/etc/rsyncd.conf

goldcome 4788 4107 0 17:37 pts/4 00:00:00 grep --color=auto rsync

goldcome@goldcome-virtual-machine:~$ sudo kill -9 4781 (结束进程)

goldcome@goldcome-virtual-machine:~$ sudo rsync --daemon --config=/etc/rsyncd.conf (重启进程)

```

######Rsync服务端口873查看

```

goldcome@goldcome-virtual-machine:~$ netstat -an |grep 873

tcp 0 0 0.0.0.0:873 0.0.0.0:* LISTEN

tcp6 0 0 :::873 :::* LISTEN

unix 3 [ ] 流 已连接 11873

```

######客户端的访问

```
rsync -arv --progress --delete --password-file=/cygdrive/g/rsync.pwd rsync://goldcome@192.168.176.152/test /cygdrive/g/Dropbox

````

#####自动同步实现

* [参考文章] (http://localhost:9002/pages/2013-07-13-rsync%E4%BD%BF%E7%94%A8%E6%80%BB%E7%BB%93.md.html#)

* TODO整理windows下自动同步


#### Rsync 完整参数列表

````

-v, --verbose 详细模式输出

-q, --quiet 精简输出模式

-c, --checksum 打开校验开关，强制对文件传输进行校验

-a, --archive 归档模式，表示以递归方式传输文件，并保持所有文件属性，等于-rlptgoD

-r, --recursive 对子目录以递归模式处理

-R, --relative 使用相对路径信息

-b, --backup 创建备份，也就是对于目的已经存在有同样的文件名时，将老的文件重新命名为~filename。可以使用--suffix选项来指定不同的备份文件前缀。

--backup-dir 将备份文件(如~filename)存放在在目录下。

-suffix=SUFFIX 定义备份文件前缀

-u, --update 仅仅进行更新，也就是跳过所有已经存在于DST，并且文件时间晚于要备份的文件。(不覆盖更新的文件)

-l, --links 保留软链结

-L, --copy-links 想对待常规文件一样处理软链结

--copy-unsafe-links 仅仅拷贝指向SRC路径目录树以外的链结

--safe-links 忽略指向SRC路径目录树以外的链结

-H, --hard-links 保留硬链结 -p, --perms 保持文件权限

-o, --owner 保持文件属主信息 -g, --group 保持文件属组信息

-D, --devices 保持设备文件信息 -t, --times 保持文件时间信息

-S, --sparse 对稀疏文件进行特殊处理以节省DST的空间

-n, --dry-run现实哪些文件将被传输

-W, --whole-file 拷贝文件，不进行增量检测

-x, --one-file-system 不要跨越文件系统边界

-B, --block-size=SIZE 检验算法使用的块尺寸，默认是700字节

-e, --rsh=COMMAND 指定使用rsh、ssh方式进行数据同步

--rsync-path=PATH 指定远程服务器上的rsync命令所在路径信息

-C, --cvs-exclude 使用和CVS一样的方法自动忽略文件，用来排除那些不希望传输的文件

--existing 仅仅更新那些已经存在于DST的文件，而不备份那些新创建的文件

--delete 删除那些DST中SRC没有的文件

--delete-excluded 同样删除接收端那些被该选项指定排除的文件

--delete-after 传输结束以后再删除

--ignore-errors 及时出现IO错误也进行删除

--max-delete=NUM 最多删除NUM个文件

--partial 保留那些因故没有完全传输的文件，以是加快随后的再次传输

--force 强制删除目录，即使不为空

--numeric-ids 不将数字的用户和组ID匹配为用户名和组名

--timeout=TIME IP超时时间，单位为秒

-I, --ignore-times 不跳过那些有同样的时间和长度的文件

--size-only 当决定是否要备份文件时，仅仅察看文件大小而不考虑文件时间

--modify-window=NUM 决定文件是否时间相同时使用的时间戳窗口，默认为0

-T --temp-dir=DIR 在DIR中创建临时文件

--compare-dest=DIR 同样比较DIR中的文件来决定是否需要备份

-P 等同于 --partial

--progress 显示备份过程

-z, --compress 对备份的文件在传输时进行压缩处理

--exclude=PATTERN 指定排除不需要传输的文件模式

--include=PATTERN 指定不排除而需要传输的文件模式

--exclude-from=FILE 排除FILE中指定模式的文件

--include-from=FILE 不排除FILE指定模式匹配的文件

--version 打印版本信息

--address 绑定到特定的地址

--config=FILE 指定其他的配置文件，不使用默认的rsyncd.conf文件

--port=PORT 指定其他的rsync服务端口

--blocking-io 对远程shell使用阻塞IO

-stats 给出某些文件的传输状态

--progress 在传输时现实传输过程

--log-format=formAT 指定日志文件格式

--password-file=FILE 从FILE中得到密码

--bwlimit=KBPS 限制I/O带宽，KBytes per second -h, --help 显示帮助信息

```

### [新增test有链接](test)

### 新增`test`有其他代码